/**
 * name: @tresjs/post-processing
 * version: v0.7.0
 * (c) 2023
 * description: Post-processing library for TresJS
 * author: Alvaro Saburido <hola@alvarosaburido.dev> (https://github.com/alvarosabu/)
 */
import { inject as X, shallowRef as w, watchEffect as E, onUnmounted as M, watch as g, defineComponent as f, provide as Y, computed as N, renderSlot as V } from "vue";
import { EffectPass as U, BloomEffect as y, DepthOfFieldEffect as F, EffectComposer as P, RenderPass as q, NormalPass as H, DepthDownsamplingPass as I, GlitchEffect as S, GlitchMode as K, OutlineEffect as R, PixelationEffect as O, VignetteTechnique as J, BlendFunction as L, VignetteEffect as D, NoiseEffect as T } from "postprocessing";
import { useTresContext as h, useRenderLoop as Q, normalizeColor as C } from "@tresjs/core";
import { HalfFloatType as z } from "three";
const $ = Symbol(), m = (l) => {
  const t = X($), e = w(null), n = w(null), { scene: o, camera: a } = h();
  E(() => {
    !a.value || !(n != null && n.value) || (n.value.mainCamera = a.value);
  });
  let s = () => {
  };
  return s = E(() => {
    !a.value || !(t != null && t.value) || !o.value || (s(), !n.value && (n.value = l(), e.value = new U(a.value, n.value), t.value.addPass(e.value)));
  }), M(() => {
    var r, i, d;
    e.value && ((r = t == null ? void 0 : t.value) == null || r.removePass(e.value)), (i = n.value) == null || i.dispose(), (d = e.value) == null || d.dispose();
  }), {
    pass: e,
    effect: n
  };
}, G = /([^[.\]])+/g, Z = (l, t) => {
  if (!t)
    return;
  const e = Array.isArray(t) ? t : t.match(G);
  return e == null ? void 0 : e.reduce((n, o) => n && n[o], l);
}, k = (l, t, e) => {
  const n = Array.isArray(t) ? t : t.match(G);
  n && n.reduce((o, a, s) => (o[a] === void 0 && (o[a] = {}), s === n.length - 1 && (o[a] = e), o[a]), l);
}, x = (l, t) => {
  const e = { ...l };
  return t.forEach((n) => delete e[n]), e;
}, W = (l, t, e, n, o = {}) => g(
  l,
  (a) => {
    var s;
    if (t.value)
      if (a === void 0) {
        const r = n();
        k(t.value, e, Z(r, e)), (s = r.dispose) == null || s.call(r);
      } else
        k(t.value, e, l());
  },
  o
), B = (l, t, e) => l.map(([n, o]) => W(
  n,
  t,
  o,
  e
)), _ = (l, t, e) => Object.keys(l).map((n) => W(
  () => l[n],
  t,
  n,
  e
)), le = /* @__PURE__ */ f({
  __name: "Bloom",
  props: {
    blendFunction: {},
    intensity: {},
    kernelSize: {},
    luminanceThreshold: {},
    luminanceSmoothing: {},
    mipmapBlur: { type: Boolean, default: void 0 }
  },
  setup(l, { expose: t }) {
    const e = l, { pass: n, effect: o } = m(() => new y(e));
    return t({ pass: n, effect: o }), B(
      [
        // blendFunction is not updated, because it has no setter in BloomEffect
        [() => e.intensity, "intensity"],
        [() => e.kernelSize, "kernelSize"],
        [() => e.luminanceSmoothing, "luminanceMaterial.smoothing"],
        [() => e.luminanceThreshold, "luminanceMaterial.threshold"]
      ],
      o,
      () => new y()
    ), (a, s) => null;
  }
}), ae = /* @__PURE__ */ f({
  __name: "DepthOfField",
  props: {
    blendFunction: {},
    worldFocusDistance: {},
    worldFocusRange: {},
    focusDistance: {},
    focusRange: {},
    bokehScale: {},
    resolutionScale: {},
    resolutionX: {},
    resolutionY: {}
  },
  setup(l, { expose: t }) {
    const e = l, { camera: n } = h(), { pass: o, effect: a } = m(() => new F(n.value, e));
    return t({ pass: o, effect: a }), B(
      [
        // blendFunction is not updated, because it has no setter in BloomEffect
        [() => e.worldFocusDistance, "circleOfConfusionMaterial.worldFocusDistance"],
        [() => e.focusDistance, "circleOfConfusionMaterial.focusDistance"],
        [() => e.worldFocusRange, "circleOfConfusionMaterial.worldFocusRange"],
        [() => e.focusRange, "circleOfConfusionMaterial.focusRange"],
        [() => e.bokehScale, "bokehScale"],
        [() => e.resolutionScale, "blurPass.resolution.scale"],
        [() => e.resolutionX, "resolution.width"],
        [() => e.resolutionY, "resolution.height"]
      ],
      a,
      () => new F()
    ), (s, r) => null;
  }
});
let v;
function A() {
  var l;
  if (v !== void 0)
    return v;
  try {
    let t;
    const e = document.createElement("canvas");
    return v = !!(window.WebGL2RenderingContext && (t = e.getContext("webgl2"))), t && ((l = t.getExtension("WEBGL_lose_context")) == null || l.loseContext()), v;
  } catch {
    return v = !1;
  }
}
const se = /* @__PURE__ */ f({
  __name: "EffectComposer",
  props: {
    enabled: { type: Boolean, default: !0 },
    children: {},
    depthBuffer: { type: Boolean, default: void 0 },
    disableNormalPass: { type: Boolean, default: !1 },
    stencilBuffer: { type: Boolean, default: void 0 },
    resolutionScale: {},
    autoClear: { type: Boolean, default: !0 },
    multisampling: { default: 0 },
    frameBufferType: { default: z }
  },
  setup(l) {
    const t = l, { scene: e, camera: n, renderer: o, sizes: a } = h(), s = w(null);
    let r = null, i = null;
    Y($, s);
    const d = () => {
      s.value && (i = new H(e.value, n.value), i.enabled = !1, s.value.addPass(i), t.resolutionScale !== void 0 && A() && (r = new I({
        normalBuffer: i.texture,
        resolutionScale: t.resolutionScale
      }), r.enabled = !1, s.value.addPass(r)));
    }, c = N(() => {
      const u = new P(), p = {
        depthBuffer: t.depthBuffer !== void 0 ? t.depthBuffer : u.inputBuffer.depthBuffer,
        stencilBuffer: t.stencilBuffer !== void 0 ? t.stencilBuffer : u.inputBuffer.stencilBuffer,
        multisampling: A() ? 0 : t.multisampling !== void 0 ? t.multisampling : u.multisampling,
        frameBufferType: t.frameBufferType !== void 0 ? t.frameBufferType : z
      };
      return u.dispose(), p;
    }), b = () => {
      !o.value && !e.value && !n.value || (s.value = new P(o.value, c.value), s.value.addPass(new q(e.value, n.value)), t.disableNormalPass || d());
    };
    g([o, e, n, () => t.disableNormalPass], () => {
      !a.width.value || !a.height.value || b();
    }), g(() => [a.width.value, a.height.value], ([u, p]) => {
      !u && !p || (s.value ? s.value.setSize(u, p) : b());
    }, {
      immediate: !0
    });
    const { onLoop: j } = Q();
    return j(({ delta: u }) => {
      if (t.enabled && o.value && s.value && a.width.value && a.height.value) {
        const p = o.value.autoClear;
        o.value.autoClear = t.autoClear, t.stencilBuffer && !t.autoClear && o.value.clearStencil(), s.value.render(u), o.value.autoClear = p;
      }
    }), M(() => {
      var u;
      (u = s.value) == null || u.dispose();
    }), (u, p) => V(u.$slots, "default");
  }
}), re = /* @__PURE__ */ f({
  __name: "Glitch",
  props: {
    blendFunction: {},
    delay: {},
    duration: {},
    strength: {},
    mode: {},
    active: { type: Boolean },
    ratio: {},
    columns: {},
    chromaticAberrationOffset: {},
    perturbationMap: {},
    dtSize: {}
  },
  setup(l, { expose: t }) {
    const e = l, { pass: n, effect: o } = m(() => new S(e));
    return t({ pass: n, effect: o }), E(() => {
      const a = () => {
        if (e.mode !== void 0)
          return e.active === !1 ? K.DISABLED : e.mode;
        const s = new S(), r = s.mode;
        return s.dispose(), r;
      };
      o.value && (o.value.mode = a());
    }), _(
      x(e, ["active", "mode", "blendFunction"]),
      o,
      () => new S()
    ), (a, s) => null;
  }
}), ue = /* @__PURE__ */ f({
  __name: "Outline",
  props: {
    outlinedObjects: {},
    blur: { type: Boolean, default: void 0 },
    xRay: { type: Boolean, default: void 0 },
    kernelSize: {},
    pulseSpeed: {},
    resolutionX: {},
    resolutionY: {},
    edgeStrength: {},
    patternScale: {},
    multisampling: {},
    blendFunction: {},
    patternTexture: {},
    resolutionScale: {},
    hiddenEdgeColor: {},
    visibleEdgeColor: {}
  },
  setup(l, { expose: t }) {
    const e = l, n = (c) => c !== void 0 ? C(c).getHex() : void 0, { camera: o, scene: a } = h(), s = {
      blur: e.blur,
      xRay: e.xRay,
      kernelSize: e.kernelSize,
      pulseSpeed: e.pulseSpeed,
      resolutionX: e.resolutionX,
      resolutionY: e.resolutionY,
      patternScale: e.patternScale,
      edgeStrength: e.edgeStrength,
      blendFunction: e.blendFunction,
      multisampling: e.multisampling,
      patternTexture: e.patternTexture,
      resolutionScale: e.resolutionScale,
      hiddenEdgeColor: n(e.hiddenEdgeColor),
      visibleEdgeColor: n(e.visibleEdgeColor)
    }, { pass: r, effect: i } = m(() => new R(a.value, o.value, s));
    t({ pass: r, effect: i }), g(
      [() => e.outlinedObjects, i],
      // watchEffect is intentionally not used here as it would result in an endless loop
      () => {
        var c;
        (c = i.value) == null || c.selection.set(e.outlinedObjects || []);
      },
      {
        immediate: !0
      }
    );
    const d = N(() => ({
      hiddenEdgeColor: e.hiddenEdgeColor ? C(e.hiddenEdgeColor) : void 0,
      visibleEdgeColor: e.visibleEdgeColor ? C(e.visibleEdgeColor) : void 0
    }));
    return B(
      [
        /* some properties are not updated because of different reasons:
            resolutionX - has no setter in OutlineEffect
            resolutionY - has no setter in OutlineEffect
            blendFunction - has no setter in OutlineEffect
            patternTexture - different type in constructor and in setter
            resolutionScale - has no setter in OutlineEffect
          */
        [() => e.blur, "blur"],
        [() => e.xRay, "xRay"],
        [() => e.pulseSpeed, "pulseSpeed"],
        [() => e.kernelSize, "kernelSize"],
        [() => e.edgeStrength, "edgeStrength"],
        [() => e.patternScale, "patternScale"],
        [() => e.multisampling, "multisampling"],
        [() => d.value.hiddenEdgeColor, "hiddenEdgeColor"],
        [() => d.value.visibleEdgeColor, "visibleEdgeColor"]
      ],
      i,
      () => new R()
    ), (c, b) => null;
  }
}), ie = /* @__PURE__ */ f({
  __name: "Pixelation",
  props: {
    granularity: {}
  },
  setup(l, { expose: t }) {
    const e = l, { pass: n, effect: o } = m(() => new O(e.granularity));
    return t({ pass: n, effect: o }), _(
      e,
      o,
      () => new O()
    ), (a, s) => null;
  }
}), ce = /* @__PURE__ */ f({
  __name: "Vignette",
  props: {
    technique: { default: J.DEFAULT },
    blendFunction: { default: L.NORMAL },
    offset: { default: 0.5 },
    darkness: { default: 0.5 }
  },
  setup(l, { expose: t }) {
    const e = l, { pass: n, effect: o } = m(() => new D(e));
    return t({ pass: n, effect: o }), _(
      x(e, ["blendFunction"]),
      o,
      () => new D()
    ), (a, s) => null;
  }
}), fe = /* @__PURE__ */ f({
  __name: "Noise",
  props: {
    premultiply: { type: Boolean, default: !1 },
    blendFunction: { default: L.SCREEN }
  },
  setup(l, { expose: t }) {
    const e = l, { pass: n, effect: o } = m(() => new T(e));
    return t({ pass: n, effect: o }), _(
      x(e, ["blendFunction"]),
      o,
      () => new T()
    ), (a, s) => null;
  }
});
export {
  le as Bloom,
  ae as DepthOfField,
  se as EffectComposer,
  re as Glitch,
  fe as Noise,
  ue as Outline,
  ie as Pixelation,
  ce as Vignette
};
